@@ -245,9 +245,10 @@ static void split_ao_device(void *tmp, char *opt, char **out_ao, char **out_dev)
         return;
     if (!opt[0] || strcmp(opt, "auto") == 0)
         return;
-    // Split on "/". If there's no "/", leave out_device NULL.
+    // Split on "/". If "/" is the final character, or absent, out_dev is NULL.
     bstr b_dev, b_ao;
-    if (bstr_split_tok(bstr0(opt), "/", &b_ao, &b_dev))
+    bstr_split_tok(bstr0(opt), "/", &b_ao, &b_dev);
+    if (b_dev.len > 0)
         *out_dev = bstrto0(tmp, b_dev);
     *out_ao = bstrto0(tmp, b_ao);
 }
@@ -454,7 +455,7 @@ bool ao_chmap_sel_adjust2(struct ao *ao, const struct mp_chmap_sel *s,
             if (!mp_chmap_equals(&res, &(struct mp_chmap)MP_CHMAP_INIT_MONO) &&
                 !mp_chmap_equals(&res, &(struct mp_chmap)MP_CHMAP_INIT_STEREO))
             {
-                MP_WARN(ao, "Disabling multichannel output.\n");
+                MP_VERBOSE(ao, "Disabling multichannel output.\n");
                 *map = (struct mp_chmap)MP_CHMAP_INIT_STEREO;
             }
         }
@@ -605,8 +606,8 @@ void ao_device_list_add(struct ao_device_list *list, struct ao *ao,
             c.desc = "Default";
         }
     }
-    c.name = c.name[0] ? talloc_asprintf(list, "%s/%s", dname, c.name)
-                       : talloc_strdup(list, dname);
+    c.name = (c.name && c.name[0]) ? talloc_asprintf(list, "%s/%s", dname, c.name)
+                                   : talloc_strdup(list, dname);
     c.desc = talloc_strdup(list, c.desc);
     MP_TARRAY_APPEND(list, list->devices, list->num_devices, c);
 }
