@@ -42,10 +42,10 @@
 #include <libavdevice/avdevice.h>
 #endif
 
-#if HAVE_LIBAVRESAMPLE
+#if HAVE_IS_LIBAV
 #include <libavresample/avresample.h>
 #endif
-#if HAVE_LIBSWRESAMPLE
+#if HAVE_IS_FFMPEG
 #include <libswresample/swresample.h>
 #endif
 
@@ -197,10 +197,10 @@ bool print_libav_versions(struct mp_log *log, int v)
         {"libavformat",   LIBAVFORMAT_VERSION_INT,   avformat_version()},
         {"libswscale",    LIBSWSCALE_VERSION_INT,    swscale_version()},
         {"libavfilter",   LIBAVFILTER_VERSION_INT,   avfilter_version()},
-#if HAVE_LIBAVRESAMPLE
+#if HAVE_IS_LIBAV
         {"libavresample", LIBAVRESAMPLE_VERSION_INT, avresample_version()},
 #endif
-#if HAVE_LIBSWRESAMPLE
+#if HAVE_IS_FFMPEG
         {"libswresample", LIBSWRESAMPLE_VERSION_INT, swresample_version()},
 #endif
     };
