@@ -61,6 +61,7 @@
 #define OPT_BASE_STRUCT struct demux_lavf_opts
 struct demux_lavf_opts {
     int probesize;
+    int probeinfo;
     int probescore;
     float analyzeduration;
     int buffersize;
@@ -77,6 +78,8 @@ struct demux_lavf_opts {
 const struct m_sub_options demux_lavf_conf = {
     .opts = (const m_option_t[]) {
         OPT_INTRANGE("demuxer-lavf-probesize", probesize, 0, 32, INT_MAX),
+        OPT_CHOICE("demuxer-lavf-probe-info", probeinfo, 0,
+                   ({"no", 0}, {"yes", 1}, {"auto", -1})),
         OPT_STRING("demuxer-lavf-format", format, 0),
         OPT_FLOATRANGE("demuxer-lavf-analyzeduration", analyzeduration, 0,
                        0, 3600),
@@ -100,6 +103,7 @@ const struct m_sub_options demux_lavf_conf = {
     },
     .size = sizeof(struct demux_lavf_opts),
     .defaults = &(const struct demux_lavf_opts){
+        .probeinfo = -1,
         .allow_mimetype = 1,
         .hacks = 1,
         // AVPROBE_SCORE_MAX/4 + 1 is the "recommended" limit. Below that, the
@@ -116,6 +120,7 @@ struct format_hack {
     const char *mime_type;
     int probescore;
     float analyzeduration;
+    bool skipinfo : 1;          // skip avformat_find_stream_info()
     unsigned int if_flags;      // additional AVInputFormat.flags flags
     bool max_probe : 1;         // use probescore only if max. probe size reached
     bool ignore : 1;            // blacklisted
@@ -128,6 +133,7 @@ struct format_hack {
     // segment, with e.g. HLS, player knows about the playlist main file only).
     bool clear_filepos : 1;
     bool ignore_start : 1;
+    bool fix_editlists : 1;
 };
 
 #define BLACKLIST(fmt) {fmt, .ignore = true}
@@ -143,10 +149,13 @@ static const struct format_hack format_hacks[] = {
     {"mp3", "audio/mpeg", 24, 0.5},
     {"mp3", NULL,         24, .max_probe = true},
 
-    {"hls", .no_stream = true, .clear_filepos = true},
+    {"hls", .no_stream = true, .clear_filepos = true, .skipinfo = true},
     {"mpeg", .use_stream_ids = true},
     {"mpegts", .use_stream_ids = true},
 
+    {"mp4", .skipinfo = true, .fix_editlists = true},
+    {"matroska", .skipinfo = true},
+
     // In theory, such streams might contain timestamps, but virtually none do.
     {"h264", .if_flags = AVFMT_NOTIMESTAMPS },
     {"hevc", .if_flags = AVFMT_NOTIMESTAMPS },
@@ -867,6 +876,9 @@ static int demux_open_lavf(demuxer_t *demuxer, enum demux_check check)
 
     guess_and_set_vobsub_name(demuxer, &dopts);
 
+    if (priv->format_hack.fix_editlists)
+        av_dict_set(&dopts, "advanced_editlist", "0", 0);
+
     avfc->interrupt_callback = (AVIOInterruptCB){
         .callback = interrupt_cb,
         .opaque = demuxer,
@@ -889,15 +901,19 @@ static int demux_open_lavf(demuxer_t *demuxer, enum demux_check check)
 
     priv->avfc = avfc;
 
-    if (!demuxer->params || !demuxer->params->skip_lavf_probing) {
+    bool probeinfo = lavfdopts->probeinfo < 0 ?
+                            !priv->format_hack.skipinfo : lavfdopts->probeinfo;
+    if (demuxer->params && demuxer->params->skip_lavf_probing)
+        probeinfo = false;
+    if (probeinfo) {
         if (avformat_find_stream_info(avfc, NULL) < 0) {
             MP_ERR(demuxer, "av_find_stream_info() failed\n");
             return -1;
         }
-    }
 
-    MP_VERBOSE(demuxer, "avformat_find_stream_info() finished after %"PRId64
-               " bytes.\n", stream_tell(priv->stream));
+        MP_VERBOSE(demuxer, "avformat_find_stream_info() finished after %"PRId64
+                   " bytes.\n", stream_tell(priv->stream));
+    }
 
     for (int i = 0; i < avfc->nb_chapters; i++) {
         AVChapter *c = avfc->chapters[i];
@@ -1027,8 +1043,25 @@ static int demux_lavf_control(demuxer_t *demuxer, int cmd, void *arg)
 
     switch (cmd) {
     case DEMUXER_CTRL_GET_TIME_LENGTH:
-        if (priv->avfc->duration == 0 || priv->avfc->duration == AV_NOPTS_VALUE)
-            return DEMUXER_CTRL_DONTKNOW;
+        if (priv->avfc->duration <= 0) {
+            double total_duration = 0;
+            double av_duration = 0;
+            for (int n = 0; n < priv->avfc->nb_streams; n++) {
+                AVStream *st = priv->avfc->streams[n];
+                if (st->duration <= 0)
+                    continue;
+                double f_duration = st->duration * av_q2d(st->time_base);
+                total_duration = MPMAX(total_duration, f_duration);
+                if (st->codecpar->codec_type == AVMEDIA_TYPE_AUDIO ||
+                    st->codecpar->codec_type == AVMEDIA_TYPE_VIDEO)
+                    av_duration = MPMAX(av_duration, f_duration);
+            }
+            double duration = av_duration > 0 ? av_duration : total_duration;
+            if (duration <= 0)
+                return DEMUXER_CTRL_DONTKNOW;
+            *(double *)arg = duration;
+            return DEMUXER_CTRL_OK;
+        }
 
         *((double *)arg) = (double)priv->avfc->duration / AV_TIME_BASE;
         return DEMUXER_CTRL_OK;
