@@ -22,13 +22,12 @@
 #include "input/keycodes.h"
 
 struct input_ctx;
+struct mpv_handle;
 
 void cocoa_put_key(int keycode);
 void cocoa_put_key_with_modifiers(int keycode, int modifiers);
 void cocoa_put_key_event(void *event);
 
-void cocoa_start_event_monitor(void);
-
 void cocoa_init_apple_remote(void);
 void cocoa_uninit_apple_remote(void);
 
@@ -36,5 +35,6 @@ void cocoa_init_media_keys(void);
 void cocoa_uninit_media_keys(void);
 
 void cocoa_set_input_context(struct input_ctx *input_context);
+void cocoa_set_mpv_handle(struct mpv_handle *ctx);
 
 #endif
