@@ -239,6 +239,12 @@ typedef struct MPContext {
     struct mp_dispatch_queue *dispatch;
     struct mp_cancel *playback_abort;
     bool in_dispatch;
+    // Number of asynchronous tasks that still need to finish until MPContext
+    // destruction is ok. It's implied that the async tasks call
+    // mp_wakeup_core() each time this is decremented.
+    // As using an atomic+wakeup would be racy, this is a normal integer, and
+    // mp_dispatch_lock must be called to change it.
+    int64_t outstanding_async;
 
     struct mp_log *statusline;
     struct osd_state *osd;
@@ -409,7 +415,10 @@ typedef struct MPContext {
     int last_chapter_seek;
     double last_chapter_pts;
 
-    bool paused;
+    bool paused;            // internal pause state
+    bool playback_active;   // not paused, restarting, loading, unloading
+    bool in_playloop;
+
     // step this many frames, then pause
     int step_frames;
     // Counted down each frame, stop playback if 0 is reached. (-1 = disable)
@@ -552,8 +561,9 @@ void mp_wakeup_core_cb(void *ctx);
 void mp_process_input(struct MPContext *mpctx);
 double get_relative_time(struct MPContext *mpctx);
 void reset_playback_state(struct MPContext *mpctx);
-void pause_player(struct MPContext *mpctx);
-void unpause_player(struct MPContext *mpctx);
+void set_pause_state(struct MPContext *mpctx, bool user_pause);
+void update_internal_pause_state(struct MPContext *mpctx);
+void update_core_idle_state(struct MPContext *mpctx);
 void add_step_frame(struct MPContext *mpctx, int dir);
 void queue_seek(struct MPContext *mpctx, enum seek_type type, double amount,
                 enum seek_precision exact, int flags);
