@@ -56,7 +56,8 @@
 // mp_wait_events() was called.
 void mp_wait_events(struct MPContext *mpctx)
 {
-    if (mpctx->sleeptime > 0)
+    bool sleeping = mpctx->sleeptime > 0;
+    if (sleeping)
         MP_STATS(mpctx, "start sleep");
 
     mpctx->in_dispatch = true;
@@ -66,7 +67,7 @@ void mp_wait_events(struct MPContext *mpctx)
     mpctx->in_dispatch = false;
     mpctx->sleeptime = INFINITY;
 
-    if (mpctx->sleeptime > 0)
+    if (sleeping)
         MP_STATS(mpctx, "end sleep");
 }
 
@@ -119,58 +120,70 @@ double get_relative_time(struct MPContext *mpctx)
     return delta * 0.000001;
 }
 
-void pause_player(struct MPContext *mpctx)
+void update_core_idle_state(struct MPContext *mpctx)
 {
-    mpctx->opts->pause = 1;
+    bool eof = mpctx->video_status == STATUS_EOF &&
+               mpctx->audio_status == STATUS_EOF;
+    bool active = !mpctx->paused && mpctx->restart_complete && mpctx->playing &&
+                  mpctx->in_playloop && !eof;
 
-    update_screensaver_state(mpctx);
+    if (mpctx->playback_active != active) {
+        mpctx->playback_active = active;
 
-    if (mpctx->paused)
-        goto end;
-    mpctx->paused = true;
-    mpctx->step_frames = 0;
-    mpctx->time_frame -= get_relative_time(mpctx);
-    mpctx->osd_function = 0;
-    mpctx->osd_force_update = true;
-    mpctx->paused_for_cache = false;
+        update_screensaver_state(mpctx);
 
-    if (mpctx->ao && mpctx->ao_chain)
-        ao_pause(mpctx->ao);
-    if (mpctx->video_out)
-        vo_set_paused(mpctx->video_out, true);
-
-    mp_wakeup_core(mpctx);
-
-end:
-    mp_notify(mpctx, mpctx->opts->pause ? MPV_EVENT_PAUSE : MPV_EVENT_UNPAUSE, 0);
+        mp_notify(mpctx, MP_EVENT_CORE_IDLE, NULL);
+    }
 }
 
-void unpause_player(struct MPContext *mpctx)
+// The value passed here is the new value for mpctx->opts->pause
+void set_pause_state(struct MPContext *mpctx, bool user_pause)
 {
-    mpctx->opts->pause = 0;
+    struct MPOpts *opts = mpctx->opts;
+    bool send_update = false;
 
-    update_screensaver_state(mpctx);
+    if (opts->pause != user_pause)
+        send_update = true;
+    opts->pause = user_pause;
 
-    if (!mpctx->paused)
-        goto end;
-    // Don't actually unpause while cache is loading.
-    if (mpctx->paused_for_cache)
-        goto end;
-    mpctx->paused = false;
-    mpctx->osd_function = 0;
-    mpctx->osd_force_update = true;
+    bool internal_paused = opts->pause || mpctx->paused_for_cache;
+    if (internal_paused != mpctx->paused) {
+        mpctx->paused = internal_paused;
+        send_update = true;
 
-    if (mpctx->ao && mpctx->ao_chain)
-        ao_resume(mpctx->ao);
-    if (mpctx->video_out)
-        vo_set_paused(mpctx->video_out, false);
+        if (mpctx->ao && mpctx->ao_chain) {
+            if (internal_paused) {
+                ao_pause(mpctx->ao);
+            } else {
+                ao_resume(mpctx->ao);
+            }
+        }
 
-    mp_wakeup_core(mpctx);
+        if (mpctx->video_out)
+            vo_set_paused(mpctx->video_out, internal_paused);
 
-    (void)get_relative_time(mpctx);     // ignore time that passed during pause
+        mpctx->osd_function = 0;
+        mpctx->osd_force_update = true;
 
-end:
-    mp_notify(mpctx, mpctx->opts->pause ? MPV_EVENT_PAUSE : MPV_EVENT_UNPAUSE, 0);
+        mp_wakeup_core(mpctx);
+
+        if (internal_paused) {
+            mpctx->step_frames = 0;
+            mpctx->time_frame -= get_relative_time(mpctx);
+        } else {
+            (void)get_relative_time(mpctx); // ignore time that passed during pause
+        }
+    }
+
+    update_core_idle_state(mpctx);
+
+    if (send_update)
+        mp_notify(mpctx, opts->pause ? MPV_EVENT_PAUSE : MPV_EVENT_UNPAUSE, 0);
+}
+
+void update_internal_pause_state(struct MPContext *mpctx)
+{
+    set_pause_state(mpctx, mpctx->opts->pause);
 }
 
 void update_screensaver_state(struct MPContext *mpctx)
@@ -178,10 +191,9 @@ void update_screensaver_state(struct MPContext *mpctx)
     if (!mpctx->video_out)
         return;
 
-    bool saver_state = mpctx->opts->pause || !mpctx->opts->stop_screensaver ||
-                       !mpctx->playback_initialized;
-    vo_control(mpctx->video_out, saver_state ? VOCTRL_RESTORE_SCREENSAVER
-                                             : VOCTRL_KILL_SCREENSAVER, NULL);
+    bool saver_state = !mpctx->playback_active || !mpctx->opts->stop_screensaver;
+    vo_control_async(mpctx->video_out, saver_state ? VOCTRL_RESTORE_SCREENSAVER
+                                                   : VOCTRL_KILL_SCREENSAVER, NULL);
 }
 
 void add_step_frame(struct MPContext *mpctx, int dir)
@@ -190,11 +202,11 @@ void add_step_frame(struct MPContext *mpctx, int dir)
         return;
     if (dir > 0) {
         mpctx->step_frames += 1;
-        unpause_player(mpctx);
+        set_pause_state(mpctx, false);
     } else if (dir < 0) {
         if (!mpctx->hrseek_active) {
             queue_seek(mpctx, MPSEEK_BACKSTEP, 0, MPSEEK_VERY_EXACT, 0);
-            pause_player(mpctx);
+            set_pause_state(mpctx, true);
         }
     }
 }
@@ -230,6 +242,8 @@ void reset_playback_state(struct MPContext *mpctx)
 #if HAVE_ENCODING
     encode_lavc_discontinuity(mpctx->encode_lavc_ctx);
 #endif
+
+    update_core_idle_state(mpctx);
 }
 
 static void mp_seek(MPContext *mpctx, struct seek_params seek)
@@ -586,7 +600,6 @@ static void handle_osd_redraw(struct MPContext *mpctx)
     if (!want_redraw)
         return;
     vo_redraw(mpctx->video_out);
-    mp_wakeup_core(mpctx);
 }
 
 static void handle_pause_on_low_cache(struct MPContext *mpctx)
@@ -605,8 +618,9 @@ static void handle_pause_on_low_cache(struct MPContext *mpctx)
     demux_control(mpctx->demuxer, DEMUXER_CTRL_GET_READER_STATE, &s);
 
     int cache_buffer = 100;
+    bool use_pause_on_low_cache = c.size > 0 || mpctx->demuxer->is_network;
 
-    if (mpctx->restart_complete && c.size > 0) {
+    if (mpctx->restart_complete && use_pause_on_low_cache) {
         if (mpctx->paused && mpctx->paused_for_cache) {
             if (!s.underrun && (!opts->cache_pausing || s.idle ||
                                 s.ts_duration >= mpctx->cache_wait_time))
@@ -618,17 +632,14 @@ static void handle_pause_on_low_cache(struct MPContext *mpctx)
                     mpctx->cache_wait_time /= 1.5 - 0.1;
                 }
                 mpctx->paused_for_cache = false;
-                if (!opts->pause)
-                    unpause_player(mpctx);
+                update_internal_pause_state(mpctx);
                 force_update = true;
             }
             mp_set_timeout(mpctx, 0.2);
         } else {
             if (opts->cache_pausing && s.underrun) {
-                bool prev_paused_user = opts->pause;
-                pause_player(mpctx);
                 mpctx->paused_for_cache = true;
-                opts->pause = prev_paused_user;
+                update_internal_pause_state(mpctx);
                 mpctx->cache_stop_time = now;
                 force_update = true;
             }
@@ -762,7 +773,7 @@ static void handle_sstep(struct MPContext *mpctx)
         if (mpctx->max_frames >= 0 && !mpctx->stop_play)
             mpctx->stop_play = AT_END_OF_FILE; // force EOF even if audio left
         if (mpctx->step_frames > 0 && !mpctx->paused)
-            pause_player(mpctx);
+            set_pause_state(mpctx, true);
     }
 }
 
@@ -830,8 +841,8 @@ static void handle_keep_open(struct MPContext *mpctx)
                 seek_to_last_frame(mpctx);
             mpctx->playback_pts = mpctx->last_vo_pts;
         }
-        if (!mpctx->opts->pause)
-            pause_player(mpctx);
+        if (opts->keep_open_pause)
+            set_pause_state(mpctx, true);
     }
 }
 
@@ -986,6 +997,7 @@ static void handle_playback_restart(struct MPContext *mpctx)
         mpctx->audio_allow_second_chance_seek = false;
         handle_playback_time(mpctx);
         mp_notify(mpctx, MPV_EVENT_PLAYBACK_RESTART, NULL);
+        update_core_idle_state(mpctx);
         if (!mpctx->playing_msg_shown) {
             if (opts->playing_msg && opts->playing_msg[0]) {
                 char *msg =
@@ -1109,8 +1121,7 @@ void run_playloop(struct MPContext *mpctx)
 
     handle_sstep(mpctx);
 
-    if (mpctx->stop_play == AT_END_OF_FILE && mpctx->seek.type)
-        mpctx->stop_play = KEEP_PLAYING;
+    update_core_idle_state(mpctx);
 
     if (mpctx->stop_play)
         return;
