@@ -74,6 +74,9 @@
                                      userInfo:nil] autorelease];
 
     [self addTrackingArea:self.tracker];
+
+    if (![self containsMouseLocation])
+        [self.adapter putKey:MP_KEY_MOUSE_LEAVE withModifiers:0];
 }
 
 - (NSPoint)mouseLocation
@@ -83,7 +86,19 @@
 
 - (BOOL)containsMouseLocation
 {
-    NSRect vF  = [[self.window screen] visibleFrame];
+    CGFloat topMargin = 0.0;
+    CGFloat menuBarHeight = [[NSApp mainMenu] menuBarHeight];
+
+    // menuBarHeight is 0 when menu bar is hidden in fullscreen
+    // 1pt to compensate of the black line beneath the menu bar
+    if ([self.adapter isInFullScreenMode] && menuBarHeight > 0) {
+        NSRect tr = [NSWindow frameRectForContentRect:CGRectZero
+                                            styleMask:NSWindowStyleMaskTitled];
+        topMargin = tr.size.height + 1 + menuBarHeight;
+    }
+
+    NSRect vF  = [[self.window screen] frame];
+    vF.size.height -= topMargin;
     NSRect vFW = [self.window convertRectFromScreen:vF];
     NSRect vFV = [self convertRect:vFW fromView:nil];
     NSPoint pt = [self convertPoint:[self mouseLocation] fromView:nil];
@@ -109,16 +124,6 @@
 
 - (BOOL)resignFirstResponder { return YES; }
 
-- (void)keyDown:(NSEvent *)event
-{
-    [self.adapter putKeyEvent:event];
-}
-
-- (void)keyUp:(NSEvent *)event
-{
-    [self.adapter putKeyEvent:event];
-}
-
 - (BOOL)canHideCursor
 {
     return !self.hasMouseDown && [self containsMouseLocation]
@@ -148,8 +153,6 @@
 
     if (self.clearing)
         return;
-
-    [self signalMousePosition];
 }
 
 - (NSPoint)convertPointToPixels:(NSPoint)point
@@ -162,14 +165,6 @@
     return point;
 }
 
-- (void)signalMousePosition
-{
-    NSPoint p = [self convertPointToPixels:[self mouseLocation]];
-    p.x = MIN(MAX(p.x, 0), self.bounds.size.width-1);
-    p.y = MIN(MAX(p.y, 0), self.bounds.size.height-1);
-    [self.adapter signalMouseMovement:p];
-}
-
 - (void)signalMouseMovement:(NSEvent *)event
 {
     NSPoint p = [self convertPointToPixels:[event locationInWindow]];
@@ -273,7 +268,8 @@
         [self preciseScroll:event];
     } else {
         const int modifiers = [event modifierFlags];
-        const int mpkey = [event deltaY] > 0 ? MP_MOUSE_BTN3 : MP_MOUSE_BTN4;
+        const int mpkey = ([event deltaX] + [event deltaY]) > 0 ?
+                            MP_MOUSE_BTN3 : MP_MOUSE_BTN4;
         [self.adapter putKey:mpkey withModifiers:modifiers];
     }
 }
